#  Keyring.pm
#    - manage a GnuPG-based keyring
#
#  Copyright (C) 2000 Frank J. Tobin <ftobin@uiuc.edu>
#
#  This file is part of pgpenvelope
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, visit the following URL:
#  http://www.gnu.org/copyleft/gpl.html
#
#  $Id: Keyring.pm,v 1.3 2000/06/25 09:12:55 ftobin Exp $
#

package PGPEnvelope::Keyring;

use strict;
use English;
use Symbol;
use Carp;
use IO::File;
use IO::Handle;

use PGPEnvelope::Common;

use GnuPG::Handles;

use Class::MethodMaker
  boolean       => [ qw( is_loaded  is_secret_keyring
			 indexed_key_info_loaded
		       ) ],
  list          => [ qw( keys       indexed_key_info ) ],
  object        => [ qw( GnuPG::Interface  gnupg ) ],
  new_hash_init => 'new';


sub import_keys
{
    my ( $self, @key_ids ) = @_;
    
    my $fileno = fileno select;
    my $newfh = IO::File->new( '&' . $fileno );
    
    my $handles = GnuPG::Handles->new( stdin  => gensym,
				       stdout => $newfh ); 
    
    $self->gnupg->recv_keys( handles            => $handles,
			     gnupg_command_args => [ @key_ids ] );
    wait;
}


sub load_keys
{
    my ( $self, @key_ids ) = @_;
    
    print "processing keyring...\n";

    $self->clear_keys();
    
    my @keys = $self->is_secret_keyring()
      ? $self->gnupg->get_secret_keys( @key_ids )
	: $self->gnupg->get_public_keys( @key_ids );
    
    $self->push_keys( @keys );
    
    $self->set_is_loaded();
    $self->clear_indexed_key_info_loaded();
}


sub get_indexed_key_info
{
    my ( $self ) = @_;
    
    $self->load_indexed_key_info()
      unless $self->indexed_key_info_loaded();
      
    return $self->indexed_key_info();
}


sub get_short_indexed_key_info
{
    my ( $self ) = @_;
    
    $self->load_indexed_key_info( short => 1 )
      unless $self->indexed_key_info_loaded();
      
    return $self->indexed_key_info();
}



sub load_indexed_key_info
{
    my ( $self, %args ) = @_;
    my $short_only = $args{short} || 0;
    
    $self->clear_indexed_key_info();
    
    my @keys = $self->keys();
    my $max_index = scalar @keys + 1;
    my $blank_prefix = ' ' x ( length( " $max_index) " ) );
    
    for ( my $i = 0; $i < scalar @keys; $i++ )
    {
	my $key = $keys[$i];
	my $index = $i + 1;
	
	my $index_prefix
	  = join( '',
		  ' ' x ( length( $max_index ) - length( $index ) + 1 ),
		  "$index) " );
	
	my @user_id_strings;
	foreach my $user_id ( $key->user_ids() )
	{
	    push @user_id_strings, $user_id->user_id_string();
	}
	
	$self->push_indexed_key_info
	  ( join ( '',
		   $index_prefix,
		   'pub  ', $key->short_hex_id, ' ',
		   $key->creation_date_string(), ' ',
		   shift @user_id_strings, "\n" )
	  );
	
	unless ( $short_only )
	{
	    foreach my $user_id ( @user_id_strings )
	    {
		$self->push_indexed_key_info
		  ( join ( '', $blank_prefix,
			   'uid                      ',
			   $user_id, "\n" )
		  );
	    }
	}
	    
	
	foreach my $subkey ( $key->subkeys() )
	{
	    $self->push_indexed_key_info
	      ( join ( '', $blank_prefix,
		       'sub  ', $subkey->short_hex_id, ' ',
		       $subkey->creation_date_string(), "\n",
		     )
	      );
	}
	
	$self->push_indexed_key_info( "\n" );
    }
    
    return;
}


sub print_keys
{
    my ( $self, @key_ids ) = @_;
    
    $self->print_keys_slave( key_ids    => [ @key_ids ],
			     with_sigs  => 0,
			   );
}


sub print_keys_with_sigs
{
    my ( $self, @key_ids ) = @_;
    
    $self->print_keys_slave( key_ids    => [ @key_ids ],
			     with_sigs  => 1,
			   );
}


sub print_keys_slave
{
    my ( $self, %args ) = @_;
    
    my $key_ids   = $args{key_ids};
    my $with_sigs = $args{with_sigs} || 0;
    
    my $fileno = fileno select;
    my $newfh = '>&' . $fileno;
    
    my $handles = GnuPG::Handles->new( stdin  => gensym,
				       stdout => $newfh,
				     );
    
    my $listing_call = $with_sigs ? 'list_sigs' : 'list_public_keys';
    
    $self->gnupg->$listing_call( handles            => $handles,
				 gnupg_command_args => $key_ids );
    
    wait;
}


1;
